
=pod    

=head1 ++ Sub package: cookie
        
Cookie handling functions.

Cookies are stored in a "jar" (hash), indexed by cookie name.  The 
contents are an anonymous array:

$jar{'name'}=@( 'value', 'domain', 'path', 'expire', 'secure' )

=cut

########################################################################
=pod    

=head1 - Function: lw::cookie_read
     
Params: \%jar, \%hout
Return: $num_of_cookies_read

Read in cookies from an %hout hash (HTTP response).

=cut
sub cookie_read {
 my ($count,$jarref,$href)=(0,@_);
 local $_;

 if(!defined $$href{'Set-Cookie'}){
	return 0;}

 if(ref($$hout{'Set-Cookie'})){ # multiple headers
	foreach ($$href{'Set-Cookie'}){
		&lw::cookie_parse($jarref,$_);
		$count++; }
 } else { # single header
	&lw::cookie_parse($jarref,$$href{'Set-Cookie'});
	$count=1; }

 return $count;
}


########################################################################
=pod    

=head1 - Function: lw::cookie_parse
     
Params: \%jar, $cookie
Return: nothing

Parses the cookie into the various parts and then sets the appropriate 
values in the %jar under the name; if the cookie is blank, it will delete 
it from the jar.

=cut
sub cookie_parse {
 my ($jarref, $header)=@_;
 my ($del,$part,@parts,@construct,$cookie_name)=(0);

 @parts=split(/;/,$header);

 foreach $part (@parts){
	if($part=~/^(.+)=(.*)$/){
		my ($name,$val)=($1,$2);
		if($name=~/^domain$/i){		
			$val=~s#^http://##;
			$val=~s#/.*$##;
			$construct[1]=$val;
		} elsif($name=~/^path$/i){
			$val=~s#/$##;
			$construct[2]=$val;
		} elsif($name=~/^expires$/i){
			$construct[3]=$val;
		} else {
			$cookie_name=$name;
			if($val eq ''){		$del=1;
			} else {		$construct[0]=$val;}
		}	
	} else {
		if($part=~/secure/){
			$construct[4]=1;}
 }	}

 if($del){
  	delete $$jarref{$cookie_name} if defined $$jarref{$cookie_name};
 } else {
	$$jarref{$cookie_name}=\@construct;
 }
}


########################################################################
=pod    

=head1 - Function: lw::cookie_write
     
Params: \%jar, \%hin, $override
Return: nothing

Goes through the given jar and sets the Cookie header in %hin pending the 
correct domain and path.  If $override is true, then the domain and path
restrictions of the cookies are ignored.

Todo: factor in expire and secure.

=cut
sub cookie_write {
 my ($jarref, $hin, $override)=@_;
 my ($name,$out)=('','');

 $override=$override||0;
 $$hin{'whisker'}->{'ssl'}=$$hin{'whisker'}->{'ssl'}||0;

 foreach $name (keys %$jarref){
	next if($$hin{'whisker'}->{'ssl'}>0 && $$jarref{$name}->[4]==0);
	if($override || 
          ($$hin{'whisker'}->{'host'}=~/$$jarref{$name}->[1]$/i &&
	   $$hin{'whisker'}->{'uri'}=~/$$jarref{$name}->[2]/i)){
		$out.="$name=$$jarref{$name}->[0];";
 }	}

 $$hin{'Cookie'}=$out;
}


########################################################################
=pod    

=head1 - Function: lw::cookie_get
     
Params: \%jar, $name
Return: @elements

Fetch the named cookie from the jar, and return the components.

=cut
sub cookie_get {
 my ($jarref,$name)=@_;

 if(defined $$jarref{$name}){
	return @{$$jarref{$name}};}

 return undef;
}


########################################################################
=pod    

=head1 - Function: lw::cookie_set
     
Params: \%jar, $name, $value, $domain, $path, $expire, $secure
Return: nothing

Set the named cookie with the provided values into the %jar.

=cut
sub cookie_set {
 my ($jarref,$name,$value,$domain,$path,$expire,$secure)=@_;
 my @construct;

 return if($name eq '');
 if($value eq ''){
	delete $$jarref{$name};
	return;}
 $path=$path||'/';
 $secure=$secure||0;

 @construct=($value,$domain,$path,$expire,$secure);
 $$jarref{$name}=\@construct; 
}


########################################################################

