=pod

=head1 ++ Sub package: dav

The dav sub package implements some beta functions for WebDAV
support.  Technically all the WebDAV requests can be done with the
appropriate setup of a whisker config hash.  These are just some wrappers
that take care of the silly XML cruft.

=cut


###########################################################################
=pod

=head1 - Function: lw::dav_do_propfind
  
Params: \%hin, \%hout, @properties
Return: Nothing (modifies %hout)

lw::dav_do_propfind does a query to the host/server defined in %hin for the
specified properties (given in @properties; if left empty, 'displayname'
is used as the default property).  Results are put into %hout (the XML
WebDAV output is in $hout{'whisker'}->{'data'}).

=cut
sub dav_do_propfind {
	my ($hin,$hout,@props)=(@_);
	my %hout;

	if($props[0] eq ''){push @props,'displayname';}

	$$hin{'whisker'}->{'method'}='PROPFIND';
	$$hin{'Content-Type'}='text/xml';
	delete($$hin{'Content-Length'});
	$$hin{'whisker'}->{'data'}='<?xml version ="1.0"?>'.
		'<propfind xmlns="DAV:"><prop>';
	map { $$hin{'whisker'}->{'data'}.='<'.$_.'/>';} @props;
	$$hin{'whisker'}->{'data'}.='</prop></propfind>';

	&lw::http_do_request($hin,$hout);
}


###########################################################################
=pod

=head1 - Function: lw::dav_do_search
  
Params: \%hin, \%hout, $sql_search
Return: Nothing (modifies %hout)

lw::dav_do_search does a query to an IIS host/server defined in %hin for the
specified search.  Results are put into %hout (the XML WebDAV output is in
$hout{'whisker'}->{'data'}).

=cut
sub do_search { # this is IIS specific
	my ($hin,$hout,$sql)=($_);
	my %hout;

	$$hin{'whisker'}->{'method'}='SEARCH';
	$$hin{'Content-Type'}='text/xml';
	delete($$hin{'Content-Length'});
	$$hin{'whisker'}->{'data'}='<?xml version ="1.0"?>'.
		'<g:searchrequest xmlns:g="DAV:">'.
		'<g:sql>' . $sql . '</g:sql>'.
		'</g:searchrequest>';
	&lw::http_do_request($hin,$hout);
}


###########################################################################
=pod

=head1 - Function: lw::dav_do_mkcol
  
Params: \%hin, \%hout
Return: Nothing (modifies %hout)

lw::dav_do_mkcol makes the collection (aka directory) specified in the URI.
This function is beta and non-functional.

=cut
sub dav_do_mkcol {
	my ($rin,$rout)=(@_);
	$$rin{'whisker'}->{'method'}='MKCOL';
	delete($rin{'Content-Length'});
	&lw::http_do_request($rin,$rout);
}


###########################################################################
=pod

=head1 - Function: lw::dav_do_copy
  
Params: \%hin, \%hout, $to
Return: Nothing (modifies %hout)

lw::dav_do_copy copies the file specified in the %hin URI to the location
specified in $to.  This function is beta and considered non-operational.

=cut
sub do_copy {
	my ($rin,$rout,$to)=(@_);
	$$rin{'whisker'}->{'method'}='COPY';
	$$rin{'Destination'}='http://'.
		$$rin{'whisker'}->{'host'}.$to;
	delete($rin{'Content-Length'});
	&lw::http_do_request($rin,$rout);
}	


###########################################################################
=pod

=head1 - Function: lw::dav_do_move
  
Params: \%hin, \%hout, $to
Return: Nothing (modifies %hout)

lw::dav_do_move moves the file specified in the %hin URI to the location
specified in $to.  This function is beta and considered non-operational.

=cut
sub do_move {
	my ($rin,$rout,$to)=(@_);
	$$rin{'whisker'}->{'method'}='MOVE';
	$$rin{'Destination'}='http://'.
		$$rin{'whisker'}->{'host'}.$to;
	delete($rin{'Content-Length'});
	&lw::http_do_request($rin,$rout);
}	

###########################################################################

